function Biofilm_strength_3D
% Calculate the biofilm strength. With more rhamnose induction, more csgA can be produced. Therefore, the curli chains become longer and the probability is higher for the curlis of
% different  bacteria to connect (and thus making a stronger biofilm)
clear all; clc; close all;
%% Variables
N=5;                    % Curlis per cell
K=300;                  % Number of segments of 1 curli
l=20;                   % Length segment(nm)
L=K*l;                  % Length polymer (nm)
xi=400;                 % Persistence length (nm)
%% Position of cells
rix=[-1000 -1000 -1000 -1000 1000  1000  1000 1000];
riy=[-1000 -1000  1000 1000 -1000 -1000  1000 1000];
riz=[-1000  1000 -1000  1000  -1000  1000 -1000 1000];

radiusx=1000;
radiusy=500;
radiusz=500;
pos_x=[];               % Store the x position
pos_y=[];               % Store the y position    
pos_z=[];               % Store the z position

for i=1:length(rix)
    store_x=[];store_y=[];store_z=[];
    [store_x,store_y,store_z]=persistance3D (N,K,l,L,xi,rix(i),riy(i),riz(i));
    
    store_x = store_x';
    store_y = store_y';
    store_z = store_z';
    store_x = reshape(store_x,[],1);
    store_y = reshape(store_y,[],1);
    store_z = reshape (store_z,[],1);
    
    pos_x = [pos_x,store_x];
    pos_y = [pos_y,store_y];
    pos_z = [pos_z,store_z];
    
    [x, y, z] = ellipsoid(rix(i),riy(i),riz(i),radiusx,radiusy,radiusz,30);
    
    
%% Plot the figure
    figure (1)
    hold on
    plot3(store_x,store_y,store_z);
    surf(x, y, z)
    az = 67;el = 12;view(az,el)
    hold off
    xlabel('x direction (nm)')
    ylabel('y direction (nm)')
    zlabel('z direction (nm)')
    
end

min_distance = 30;                      % Minimal distance for curlis of different cells to connect
p_max = (length(rix)-1);
store_r=zeros(1,4);
                                        % Store R is a matrix that looks like:
                                        % [segment position;  cell nr; segment position; cell nr]

for p = 1:p_max;                        
    for k = 1:(p_max + 1 - p);          
        for j = 1:length(pos_x)         
            for i = 1:length(pos_x)  
                r = sqrt ( (pos_x(j,p) - pos_x(i,p+k)).^2 + (pos_y(j,p) - pos_y(i,p+k)).^2 + (pos_z(j,p) - pos_z(i,p+k)).^2);
                if r < min_distance
                    if (j-(store_r(end,1)))>2
                        store_r = [store_r; [j, p ,i ,p+k]];
                    end
                end
            end
        end
    end
end
Overlapping_curlis = size(store_r,1) - 1

end

function [ store_x,store_y,store_z ] = persistance3D (N,K,l,L,xi,rix,riy,riz)
store_x = [];
store_y = [];
store_z = [];

for i=1:N
    % Define the angles
    mu=0;
    sigma=sqrt(l/xi);
    angle_u=mu+sigma.*randn(K,1);
    angle_v=mu+sigma.*randn(K,1);
    
    % Cofficients
    ct= cos(angle_u).*cos(angle_v)./sqrt(1-(sin(angle_u).*sin(angle_v)).^2);
    cu= sin(angle_u).*cos(angle_v)./sqrt(1-(sin(angle_u).*sin(angle_v)).^2);
    cv= cos(angle_u).*sin(angle_v)./sqrt(1-(sin(angle_u).*sin(angle_v)).^2);
    
    % First t, u, v and r
    r=zeros(3,K);
    r(:,1)=[rix;riy;riz];
    
    t=ones(3,K);
    u=zeros(3,K);
    v=zeros(3,K);
    alfa=pi*rand;
    beta=2*pi*rand;
    t(:,1)=[sin(alfa)*cos(beta); sin(alfa)*sin(beta); cos(alfa)];
    u(:,1)=[cos(alfa)*cos(beta); cos(alfa)*sin(beta); -sin(alfa)];
    v(:,1)=[-sin(beta); cos(beta); 0];
    
    % Calculating t, u, v, r
    for j=2:K
        t(:,j)=ct(j)*t(:,j-1)+cu(j)*u(:,j-1)+cv(j)*v(:,j-1);
        alfa=acos(t(3,j));
        beta=atan2(t(2,j), t(1,j));
        u(:,j)=[cos(alfa)*cos(beta); cos(alfa)*sin(beta); -sin(alfa)];
        v(:,j)=[-sin(beta); cos(beta); 0];
        r(:,j)=r(:,j-1)+l.*t(:,j);
    end
    
    store_x= [r(1,:);store_x];
    store_y= [r(2,:);store_y];
    store_z= [r(3,:);store_z];
    
end
end

